#!/usr/bin/perl
#
################################################################################
#
# IPCop Web-Iface for GKrellM Daemon
#
# Copyright (C) 2007-2014 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
#
# $Id: gkrellmdgui.cgi 558 2014-12-15 10:51:54Z weizen_42 $
#
# 2007-07 created by weizen_42
#

# Add entry in menu
# MENUENTRY services 091 "GKrellM" "GKrellM"

use strict;

# enable only the following on debugging purpose
use warnings;
use CGI::Carp 'fatalsToBrowser';

use LWP::UserAgent;

require '/usr/lib/ipcop/general-functions.pl';
require '/usr/lib/ipcop/lang.pl';
require '/usr/lib/ipcop/header.pl';


my $version = 'v0.1.0';
my $addonname = 'gkrellmd';
my $onlineversion = '';
my $onlinelink = '';
my $timestamplastcheck = '/var/ipcop/addons/gkrellmd/lastcheck';
my $flagdonotcheck = '/var/ipcop/addons/gkrellmd/noversioncheck';
my $debug = 0;
my $errormessage = '';

my %checked = ();
my %selected=();
my %hashcgi=();
$hashcgi{'ACTION'} = '';
$hashcgi{'ENABLED'} = 'off';
my %hashsettings = ();
$hashsettings{'ENABLED'} = 'off';

&General::getcgihash(\%hashcgi);
&General::readhash('/var/ipcop/addons/gkrellmd/settings', \%hashsettings);

&Header::showhttpheaders();

&Header::openpage('GKrellM', 1, '');
&Header::openbigbox('100%', 'left', '');


if ($hashcgi{'ACTION'} eq "$Lang::tr{'save'}") {
    map ($hashsettings{$_} = $hashcgi{$_}, ('ENABLED'));

    &General::writehash('/var/ipcop/addons/gkrellmd/settings', \%hashsettings);
    &WriteConfig_gkrellmd_conf();
    system("/usr/local/bin/gkrellmdctrl --restart >/dev/null 2>&1 &");

    # Add a short delay
    sleep(2);
}
ERROREXIT:


if ($errormessage) {
    &Header::openbox('100%', 'left', "$Lang::tr{'error messages'}:", 'error');
    print "<font class='base'>$errormessage&nbsp;</font>";
    &Header::closebox();
}


#
# Check for new version
#
&checkfornewversion($addonname, $version);
if ( $onlineversion ne '' )
{
  &Header::openbox('100%', 'left', $Lang::tr{'info'}, 'warning');
  print <<END
<table width="100%"><tr>
<td>A new version is available: <a href="$onlinelink" target="_blank"><b>$onlineversion</b></a></td>
</tr></table>
END
;
  &Header::closebox();
}


$checked{'ENABLED'}{'off'} = '';
$checked{'ENABLED'}{'on'} = '';
$checked{'ENABLED'}{$hashsettings{'ENABLED'}} = "checked='checked'";

$selected{'LOGLEVEL'}{'0'} = '';
$selected{'LOGLEVEL'}{'1'} = '';
$selected{'LOGLEVEL'}{'2'} = '';
$selected{'LOGLEVEL'}{'3'} = '';
$selected{'LOGLEVEL'}{'4'} = '';
$selected{'LOGLEVEL'}{$hashsettings{'LOGLEVEL'}} = "selected='selected'";


&Header::openbox('100%', 'left', 'GKrellM');
my $sactive = &General::isrunning('gkrellmd', 'nosize');

print <<END
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<table width='100%'>
<tr>
    <td>GKrellM Daemon:</td>
    $sactive
    <td colspan='2' align='right'><b><small><a href="http://www.ban-solms.de/t/IPCop.html" target="_blank">GKrellM Addon $version</a></small></b></td>
</tr>
<tr>
    <td colspan='4'><hr /></td>
</tr><tr>
    <td width='25%' class='base'>$Lang::tr{'enabled'}:</td>
    <td width='25%'><input type='checkbox' name='ENABLED' $checked{'ENABLED'}{'on'} /></td>
    <td colspan='2'>&nbsp;</td>
</table>
<hr />
<table width='100%'>
<tr>
    <td class='comment1button'>&nbsp;</td>
    <td class='button1button'>
        <input type='submit' name='ACTION' value='$Lang::tr{'save'}' />
    </td>
    <td class='onlinehelp'>
        <a href='http://www.ban-solms.de/t/IPCop-gkrellm.html' target='_blank'>
        <img src='/images/web-support.png' alt='Online help' title='Online help' /></a>
    </td>
</tr>
</table>
</form>
END
;

&Header::closebox();
&Header::closebigbox();
&Header::closepage();


sub checkfornewversion
{
  my $addon = shift;
  my $version = shift;
  $onlineversion = '';

  if ( -e $flagdonotcheck )
  {
    return;
  }

  # only check if we are online and last check was some time ago
  if ( (! -e '/var/ipcop/red/active') || (-e $timestamplastcheck) && (int(-M $timestamplastcheck) < 5) )
  {
    return;
  }

#workaround to suppress a warning when a variable is used only once
  my @dummy = ( $General::version );
  undef (@dummy);

  my $ua = LWP::UserAgent->new;
  $ua->timeout(120);
  $ua->agent("Mozilla/4.0 (compatible; IPCop $General::version; $version)");
  my $content = $ua->get("http://www.ban-solms.de/version/$addon");

  if ( $content->is_success )
  {
    # compare the versions, format is v1.2.3
    $content->content =~ /v(\d+).(\d+).(\d+)/;
    my $ver1 = $1;
    my $ver2 = $2;
    my $ver3 = $3;

    $version =~ /v(\d+).(\d+).(\d+)/;

    if ( ($ver1 > $1) || (($ver1 == $1) && ($ver2 > $2)) || (($ver1 == $1) && ($ver2 == $2) && ($ver3 > $3)) )
    {
      $onlineversion = "v$ver1.$ver2.$ver3";

      $content->content =~ /http(.*)/;
      $onlinelink = "http$1";
    }
    else
    {
      # no news, recheck in a couple of days
      system("touch $timestamplastcheck");
    }
  }
}


sub WriteConfig_gkrellmd_conf
{
    open (CONFIGFILE, ">/var/ipcop/addons/gkrellmd/gkrellmd.conf");
    print CONFIGFILE <<END

#
# config-file for gkrellmd written by IPCop GKrellM Addon
#

# The server update frequency is independent of and should be less than
# the client update frequency.  Values may be from 1 to 10 and should be
# smaller values to reduce network traffic.
#
#update-hz 3

# Limit number of simultaneous clients allowed to connect.
#
#max-clients 2

# Specify a specific network interface to listen on for connections.
# By default gkrellmd listens on all available network interfaces.
#
#address 127.0.0.1

# Specify the port to listen on for connections.
#
port 19150

# List of hosts allowed to connect.  If no hosts are specified in a
# gkrellmd.conf file or on the command line, all hosts will be allowed.
#
#allow-host	localhost
#allow-host	127.0.0.1
#allow-host	::1
#allow-host	192.168.0.*

# Drop privileges after startup (you must start gkrellmd as root to do it).
# NOTE: Option ignored on Windows
#
#user	nobody
#group	proc

# Create a PID file for the running gkrellmd.  Default is no PID file.
# NOTE: Option ignored on Windows
#
pidfile /var/run/gkrellmd.pid

# Run in background and detach from the controlling terminal
# NOTE: Option ignored on Windows
#
detach

# Enable writing logging message to the system syslog file
# NOTE: On windows this enables logging to the windows event log
#
syslog

# Time interval between checks for various monitors.  If nfs-interval
# is <= 0 then gkrellmd will not read data for nfs file system types.
#
#fs-interval 2
#nfs-interval 16

# The Internet monitor defaults to reading tcp connections once per second.
# However, for Linux SMP kernels where reading /proc/net/tcp causes high
# cpu usage, the inet-interval may be set to 1-20 seconds to slow down
# /proc/net/tcp reads.  Or set it to 0 to totally disable the Inet monitor.
# Requires at least gkrellmd version 2.1.8.
#
#inet-interval 1

# If the mbmon daemon is started before gkrellmd with this command:
#     mbmon -r -P port-number
# (the "-r" mbmon tag mode is required) then gkrellmd will monitor mbmon
# reported sensors if this is uncommented and the port-numbers match.
#
#mbmon-port port-number

# Configure gkrellm clients to disconnect from a gkrellmd server if
# there is an io-timeout seconds interval where the client receives no input
# from the server.  Use this for conditions where gkrellmd may be
# ungracefully terminated such that socket connections are not closed.
# Minimum is 2 (less than 2 for no I/O disconnecting and is the default).
# Requires at least 2.1.8 versions of both gkrellmd and gkrellm.
#
io-timeout 15

# Configure gkrellm clients to attempt automatic reconnects to a
# gkrellmd server every reconnect-timeout seconds after a disconnected
# state is detected.  Disconnected states are created by normal gkrellmd
# shutdowns or by an expiring io-timeout.
# Minimum is 2 (less than 2 for no automatic reconnecting and is the default).
# Requires at least 2.1.8 versions of both gkrellmd and gkrellm.
#
reconnect-timeout 15

# Server side local mailbox counts can be sent to gkrellm clients.  List here
# paths to mbox, MH mail, or Maildir style mailboxes.
# Requires at least 2.1.11 versions of both gkrellmd and gkrellm.
# NOTE: Option ignored on Windows
#
#mailbox /var/mail/bill
#mailbox ~/Mail/inbox

# List of plugins to enable.  Use "gkrellmd -plist" to view a list of all
# available plugins.  Use "gkrellmd -plog" to output a log of the plugin
# install process (note: the detach option is ignored when using -plog).
#
#plugin-enable gkrelltopd
#plugin-enable gkrellmwho2d

# Configure gkrellmd to send the connect time for a network interface
# to all clients to be displayed in the client gkrellm timer button
# monitor display.  If this is done, the client gkrellm timer button can
# still execute commands on the client, but the button will not affect the
# timer display.  If you want the client timer button to execute commands
# on the server, your timer button commands can use ssh.
# So set this if your server box has a ppp or ippp connection and you
# want to monitor its connect time.
#net-timer ppp0

# For debugging purposes, gkrellmd can print out messages on console.
# There are several debug areas which can be enabled independently by 
# summing up the following values: 
#
# system              0x1
# mail                0x10
# net                 0x20
# timer               0x40
# sensors             0x80
# sensors             0x100
# (w/o libsensors)
# inet                0x800
# battery             0x8000
#
# i.e. debug-level 0x51 enables messages for timer, mail and system
#
#debug level 0x1
END
    ;

#    if ($hashsettings{'SERVER_IP'} ne '') {
#        $hashsettings{'SERVER_IP'} =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/;
#        print CONFIGFILE "allow ^$1\\.$2\\.$3\\.$4\$\n";
#    }

    close CONFIGFILE;
}
